package com.gic;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.cloud.data.hook.api.entity.FlatQueryCondition;
import com.gic.cloud.data.hook.api.entity.FlatQueryFieldType;
import com.gic.cloud.data.hook.api.entity.FlatQueryTaskCondition;
import com.gic.cloud.data.hook.service.DecryptUtils;
import com.gic.cloud.data.hook.service.DownloadFunc;
import com.gic.cloud.data.hook.service.FileUtil;
import com.opencsv.CSVReader;
import com.opencsv.CSVReaderBuilder;
import org.apache.commons.lang.time.DateUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

public class ExceTest2 {
    private static Logger logger = LoggerFactory.getLogger(ExceTest2.class);
    public static final String SAVE_FOLDER = "D:\\testorder";
    /**
     * 读物文件
     * @param dirName
     * @param func
     */
    private static void readCsvFile(FlatQueryTaskCondition condition, String dirName, DownloadFunc func){
        File file = new File(SAVE_FOLDER+"/"+dirName);
        File[] files = file.listFiles();
        List<FlatQueryCondition> titles=null;
        List<FlatQueryCondition> conditions = condition.getConditions();
        Map<String, FlatQueryCondition> columnInfoMap = conditions.stream().collect(Collectors.toMap(mid -> mid.getFieldMark(), mid ->mid));
        List<File> fileList = Arrays.stream(files).sorted(Comparator.comparing(File::getName)).collect(Collectors.toList());
        for (File midFile : fileList) {
            if (!midFile.getName().endsWith("csv")) {
                continue;
            }
            String lastcells=null;
            String[] cells=null;
            try {
                CSVReader csvReader = new CSVReaderBuilder(new FileReader(midFile)).build();
                 cells = csvReader.readNext();
                boolean first=true;
                do {
                    if (titles == null) {
                        titles=new ArrayList<>();
                        for (String cell : cells) {
                            logger.info("cell:{}",cell);
                            titles.add(columnInfoMap.get(cell));
                        }
                    }
                    if (first) {
                        first=false;
                        continue;
                    }
                    func.deal(cells,titles,first);
                    lastcells=JSONObject.toJSONString(cells);
                }while ((cells = csvReader.readNext())!=null);
                logger.info("读取结束：{}",midFile.getName());
                csvReader.close();
            } catch (Exception e) {
                logger.info("lastcell: {}",lastcells);
                logger.info("cells: {}",cells[0]);
                logger.info("cells: {},{},{}",cells[1]);
                logger.info("读取异常：{}",e);
            }
        }
    }

    /**
     * 读物文件
     * @param dirName
     * @param func
     */
    private static void readJsonFile(FlatQueryTaskCondition condition, String dirName, DownloadFunc func){
        File file = new File(SAVE_FOLDER+"/"+dirName);
        File[] files = file.listFiles();
        List<FlatQueryCondition> titles=null;
        List<FlatQueryCondition> conditions = condition.getConditions();
        List<String> keys = conditions.stream().map(mid -> mid.getFieldMark()).collect(Collectors.toList());
        Map<String, FlatQueryCondition> columnInfoMap = conditions.stream().collect(Collectors.toMap(mid -> mid.getFieldMark(), mid ->mid));
        List<File> fileList = Arrays.stream(files).sorted(Comparator.comparing(File::getName)).collect(Collectors.toList());
        for (File midFile : fileList) {
            if (!midFile.getName().endsWith("json")) {
                continue;
            }
            try {
                BufferedReader reader=new BufferedReader(new FileReader(midFile));
                boolean first=true;
                Exception exception=null;
                try {
                    String line = reader.readLine();
                    while (line != null) {
                        List<String> cellList=new ArrayList<>();
                        JSONObject jsonObject = JSONObject.parseObject(line);
                        for (String key : keys) {
                            String cellVal = jsonObject.getString(key);
                            cellList.add(cellVal);
                        }
                        String[] cells = cellList.toArray(new String[]{});
                        func.deal(cells,conditions,first);
                        first=false;
                        line = reader.readLine();
                    }
                    logger.info("读取结束：{}",midFile.getName());
                } catch (Exception e) {
                    exception=e;
                }finally {
                    reader.close();
                }
                if (exception!=null) {
                    throw exception;
                }
            } catch (Exception e) {
                logger.info("读取异常：{}",e);
                throw new RuntimeException(e);
            }
        }
    }

    public static class XlsxFileInfo{
        String filepath;
        SXSSFWorkbook workbook;
        CellStyle yyyyMMddhhmmss;
        CellStyle yyyyMMdd;
    }

    /**
     * 写入excel
     * @param originalFilePath
     * @param cells
     */
    private static void saveXlsSplitNew(String originalFilePath, String[] cells, List<FlatQueryCondition> titles, XlsxFileInfo xlsxFileInfo, AtomicInteger count, boolean endFlag, int queryDataType){
        try {
            if (xlsxFileInfo.workbook==null) {
                xlsxFileInfo.workbook=new SXSSFWorkbook(100);
                Sheet sheet = xlsxFileInfo.workbook.createSheet();
                Row row = sheet.createRow(0);
                Cell cell;
                logger.info("类型:{}",JSONObject.toJSONString(titles));
                for(int j =0; j<titles.size(); j++){
                    cell = row.createCell(j);
                    cell.setCellValue(titles.get(j).getFieldMark());
                }

                //日期
                CellStyle yyyyMMddhhmmss = xlsxFileInfo.workbook.createCellStyle();
                DataFormat dataFormat = xlsxFileInfo.workbook.createDataFormat();
                yyyyMMddhhmmss.setDataFormat(dataFormat.getFormat("yyyy-MM-dd HH:mm:ss"));
                xlsxFileInfo.yyyyMMddhhmmss=yyyyMMddhhmmss;

                //日期
                CellStyle yyyyMMdd = xlsxFileInfo.workbook.createCellStyle();
                DataFormat yyyyMMddDataFormat = xlsxFileInfo.workbook.createDataFormat();
                yyyyMMdd.setDataFormat(yyyyMMddDataFormat.getFormat("yyyy-MM-dd"));
                xlsxFileInfo.yyyyMMdd=yyyyMMdd;
            }
            count.incrementAndGet();

            SXSSFWorkbook wb = xlsxFileInfo.workbook; // 内存中保留 100 行
            if (!endFlag) {
                Sheet sheet = wb.getSheetAt(0);

                Row row = sheet.createRow(count.get());
                for (int j = 0; j < cells.length; j++) {
                    FlatQueryCondition flatQueryCondition = titles.get(j);
                    String type = titles.get(j).getFieldType();
                    Cell midCell = row.createCell(j);
                    String columnValue = cells[j];
                    if (StringUtils.isBlank(columnValue)) {
                        midCell.setCellValue(columnValue);
                        continue;
                    }
                    switch (type) {
                        case FlatQueryFieldType.DATE:
                            if (columnValue.length() == 10) {
                                midCell.setCellStyle(xlsxFileInfo.yyyyMMdd);
                                midCell.setCellValue(DateUtils.parseDate(columnValue, new String[]{"yyyy-MM-dd"}));
                            } else {
                                midCell.setCellStyle(xlsxFileInfo.yyyyMMddhhmmss);
                                midCell.setCellValue(DateUtils.parseDate(columnValue, new String[]{"yyyy-MM-dd HH:mm:ss"}));
                            }
                            break;
                        case FlatQueryFieldType.NUMBER:
                            if (columnValue.indexOf(".") > 0) {
                                midCell.setCellValue(Double.valueOf(columnValue));
                            } else {
                                midCell.setCellValue(Long.valueOf(columnValue));
                            }
                            break;
                        default:
                            midCell.setCellValue(DecryptUtils.encryptionDeal(queryDataType,columnValue,flatQueryCondition));
                    }
                }
            }
            Integer limitSize = FileUtil.getLimitSize();
            int c = count.get();
            if (c%5000==0) {
                logger.info("进度：{}-{}",originalFilePath,c);
            }
            if (c>=limitSize.intValue()||endFlag) {
                logger.info("{}-count:{}",originalFilePath,count.get());
                FileOutputStream fileOut = new FileOutputStream(originalFilePath);
                wb.write(fileOut);
                //fileOut.flush(); // SXSSFWorkbook 使用 auto-flush 模式
                fileOut.close();
                //wb.close();
                wb.dispose(); // SXSSFWorkbook 没有 close 方法
                logger.info("[ runDownloadTask.run ]: {}", "已生成自助指标下载文件 " + originalFilePath + ".xlsx");
                count.set(0);
            }
        }catch (Exception e) {
            logger.info("异常：{}",e);
            throw new RuntimeException(e);
        }

    }

    public static void main(String[] args) {
        //下载文件
        String taskId="test";
        String dirName="hdfs"+taskId;
        String conditionStr="{\"allFields\":[\"wechat_nickname\",\"add_buddy_time\",\"phone_number\",\"open_card_time\",\"clerk_name\",\"clerk_store_name\",\"main_store_name\"],\"amount\":627549,\"authStoreIdList\":[],\"buildPermitted\":\"1\",\"conditions\":[{\"enableEncrypt\":false,\"extendFilter\":\"\",\"fieldMark\":\"微信昵称\",\"fieldName\":\"wechat_nickname\",\"fieldType\":\"text\",\"filterTunnel\":\"\",\"mainFilter\":\"\"},{\"enableEncrypt\":false,\"extendFilter\":\"\",\"fieldMark\":\"添加好友时间\",\"fieldName\":\"add_buddy_time\",\"fieldType\":\"date\",\"filterTunnel\":\"\",\"mainFilter\":\"\"},{\"enableEncrypt\":true,\"extendFilter\":\"\",\"fieldMark\":\"会员手机号码\",\"fieldName\":\"phone_number\",\"fieldType\":\"text\",\"filterTunnel\":\"\",\"mainFilter\":\"\"},{\"enableEncrypt\":false,\"extendFilter\":\"\",\"fieldMark\":\"开卡时间\",\"fieldName\":\"open_card_time\",\"fieldType\":\"date\",\"filterTunnel\":\"\",\"mainFilter\":\"\"},{\"enableEncrypt\":false,\"extendFilter\":\"\",\"fieldMark\":\"好友导购姓名\",\"fieldName\":\"clerk_name\",\"fieldType\":\"text\",\"filterTunnel\":\"\",\"mainFilter\":\"\"},{\"enableEncrypt\":false,\"extendFilter\":\"\",\"fieldMark\":\"导购门店名称\",\"fieldName\":\"clerk_store_name\",\"fieldType\":\"text\",\"filterTunnel\":\"\",\"mainFilter\":\"\"},{\"enableEncrypt\":false,\"extendFilter\":\"\",\"fieldMark\":\"会员服务门店名称\",\"fieldName\":\"main_store_name\",\"fieldType\":\"text\",\"filterTunnel\":\"\",\"mainFilter\":\"\"}],\"decryptFilters\":[\"会员手机号码\"],\"enterpriseIds\":[\"ff8080815c1206fa015c1554c27701b3\"],\"execDistinct\":false,\"orderDir\":\"\",\"orderField\":\"\",\"queryDataType\":0,\"tableId\":\"extract_entwechat_detail_d\",\"taskId\":\"1682325929774\"}";
        FlatQueryTaskCondition condition = JSON.parseObject(conditionStr, FlatQueryTaskCondition.class);
        List<String> xlsxFiles=new ArrayList<>();
        AtomicInteger count=new AtomicInteger(0);
        AtomicReference<XlsxFileInfo> currentFile=new AtomicReference<>();
        readJsonFile(condition,dirName,(cells,titles,firstFlag)->{
            if (count.get()==0) {
                XlsxFileInfo xlsxFileInfo = new XlsxFileInfo();
                xlsxFileInfo.filepath=SAVE_FOLDER + "/" + taskId + xlsxFiles.size() + ".xlsx";
                currentFile.set(xlsxFileInfo);
                xlsxFiles.add(currentFile.get().filepath);
            }
            saveXlsSplitNew(currentFile.get().filepath,cells,titles,currentFile.get(),count,false,1);
        });
        saveXlsSplitNew(currentFile.get().filepath,null,null,currentFile.get(),count,true,1);
    }
}
